@@ -1,4 +1,4 @@
-/* $Id: avcheck.c,v 1.19 2002/07/27 20:22:05 mjt Exp $
+/* $Id: avcheck.c,v 1.21 2002/10/09 22:07:59 mjt Exp $
  * avcheck mail virus scanner client.
  * Michael Tokarev <mjt@corpit.ru>
  * Public domain.
@@ -439,7 +439,7 @@ scan_avp(int fd, const char *path, const char *avname) {
     /* this returned when AVP is unable to parse file structure for
        various reasons. */
   case 8: /* file is corrupted (e.g. archive) - do not treat it as bad */
-    warn("unexpected %s return code %d (0x%04x)", avname, r & 0xcf,
+    warn("unexpected %s return code %d (0x%04x) (%s)", avname, r & 0xcf, r,
          ((r & 0xcf) == 1) ? "virus scan was incomplete" : "file is corrupted");
     return 0;
   case 4: /* known virus(es) detected */
@@ -733,6 +733,7 @@ int main(int argc, char **argv) {
   char *waitfile = NULL;
   unsigned avtimeout = 0; /* timeout for antivirus daemon operation */
   char *okhdr = NULL; /* header to add for good mails */
+  int okcode = 0;
 
   char *ipath = argv[0];
   char *iscript = NULL;
@@ -762,6 +763,7 @@ Usage:  %s options -- recipient...,  where options are:\n\
  -t timeout - timeout, in secounds, to wait for antivirus\n\
    answer (default is 0, i.e. no timeout)\n\
  -n - do not send (reinject) good mail back into the mail system\n\
+ -g code - return this code for good, uninfected emails\n\
  -S sendmail_path - /path/to/sendmail-compatible executable\n\
    (possible with args -- either using multi-word value or repeating\n\
    this option to specify additional arguments) or host:port to speak\n\
@@ -781,7 +783,7 @@ Usage:  %s options -- recipient...,  where options are:\n\
     return EX_USAGE;
   }
 
-  while((c = getopt(argc, argv, "t:f:d:s:S:ni:cw:h:")) != EOF)
+  while((c = getopt(argc, argv, "t:f:d:s:S:ni:cw:h:g:")) != EOF)
     switch(c) {
     case 't': /* timeout */
       if ((c = atoi(optarg)) < 0)
@@ -809,6 +811,11 @@ Usage:  %s options -- recipient...,  where options are:\n\
       break;
     case 'd': tmpdir = optarg; break;
     case 'n': nosend = 1; break;
+    case 'g':
+      if ((c = atoi(optarg)) < 0 || c > 255)
+	err(0, "invalid code for -g option `%s'", optarg);
+      okcode = c;
+      break;
     case 'i': iscript = optarg; break;
     case 'c': mailclient = 1; break;
     case 'w': waitfile = optarg; break;
@@ -916,6 +923,7 @@ Usage:  %s options -- recipient...,  where options are:\n\
       sendit(msgfd, from, argv, argc, 1, okhdr);
     }
     cleanup();
+    return okcode;
   }
   else { /* else call infected mail handler */
     char **av, **ev;
@@ -964,7 +972,6 @@ Usage:  %s options -- recipient...,  where options are:\n\
 
     execve(av[0], av, ev);
     err(errno, "unable to execute `%s'", av[0]);
+    return 0;
   }
-
-  return 0;
 }
